/****************************************
 *  COPYRIGHT (C) 2010
 *  Holger Graf
 ****************************************/

package siarchive.galaxy;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.sql.SQLException;
import java.util.Collection;
import java.util.EnumSet;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTabbedPane;
import javax.swing.JToolTip;
import javax.swing.UIDefaults;
import javax.swing.UIManager;
import javax.swing.border.BevelBorder;

import siarchive.DataManager;
import siarchive.Resources;
import siarchive.components.Building;
import siarchive.components.Defense;
import siarchive.components.Flavor;
import siarchive.components.NoteBox;
import siarchive.components.PlanetarySystem;
import siarchive.components.Position;
import siarchive.components.Ship;
import siarchive.components.Status;
import siarchive.persistence.BattleReport;
import siarchive.persistence.Notable;
import siarchive.persistence.Planet;
import siarchive.persistence.Player;
import siarchive.persistence.SpyReport;
import siarchive.reports.BattleReportView;
import siarchive.reports.SpyReportParser;
import siarchive.reports.SpyReportView;
import siarchive.search.BattleReportSummaryPane;
import siarchive.search.PlanetSummaryPane;
import siarchive.search.SpyReportSummaryPane;

/**
 * @author graf
 *
 */
public class GalaxyComponent extends JComponent
{
    private static final long serialVersionUID = 4025295210952509026L;

    private static final int toolTipWidth = 35;

    protected GalaxyView parent;
    protected DataManager dataManager;

    private int galaxy = 1;
    private int system = 1;
    private Font baseFont;
    private Font boldFont;

    private JMenuItem popupAlliance = new JMenuItem();
    private JMenuItem popupPlayer = new JMenuItem();
    private JMenuItem popupSystem = new JMenuItem();
    private JMenuItem popupSpyReports = new JMenuItem();
    private JMenuItem popupBattleReports = new JMenuItem();
    private Planet menuPlanet = null;

    private Map<Rectangle, SpyReport> spyReports = new LinkedHashMap<Rectangle, SpyReport>();
    private Map<Rectangle, BattleReport> battleReports = new LinkedHashMap<Rectangle, BattleReport>();
    private Map<Rectangle, Notable<Planet>> notes = new LinkedHashMap<Rectangle, Notable<Planet>>();

    public GalaxyComponent( final GalaxyView parent )
    {
        super();
        this.parent = parent;
        this.dataManager = parent.getDataManager();

        setOpaque( true );

        setBorder( BorderFactory.createBevelBorder( BevelBorder.LOWERED ) );
        setToolTipText( " " );

        this.addKeyListener( new KeyAdapter()
        {
            public void keyPressed( KeyEvent event )
            {
                eventKeyPressed( event );
            }
        } );

        popupAlliance.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                actionShowAlliance( e );
            }
        } );
        popupPlayer.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                actionShowPlayer( e );
            }
        } );
        popupSystem.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                actionShowSystem( e );
            }
        } );
        popupBattleReports.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                actionShowBattleReports( e );
            }
        } );
        popupSpyReports.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                actionShowSpyReports( e );
            }
        } );

        this.addMouseListener( new MouseAdapter()
        {
            public void mouseClicked(MouseEvent e)
            {
                boolean focusRequest = true;
                if( spyReports.size() > 0 )
                {
                    for( Map.Entry<Rectangle, SpyReport> entry : spyReports.entrySet() )
                    {
                        if( entry.getKey().contains( e.getX(), e.getY() ) )
                        {
                            SpyReportView spyReportViewer = new SpyReportView( entry.getValue(), parent.getMainFrame() );
                            spyReportViewer.setVisible( true );
                            focusRequest = false;
                        }
                    }
                }
                if( battleReports.size() > 0 )
                {
                    for( Map.Entry<Rectangle, BattleReport> entry : battleReports.entrySet() )
                    {
                        if( entry.getKey().contains( e.getX(), e.getY() ) )
                        {
                            BattleReportView battleReportViewer = new BattleReportView( entry.getValue(), parent.getMainFrame() );
                            battleReportViewer.setVisible( true );
                            focusRequest = false;
                        }
                    }
                }
                if( notes.size() > 0 )
                {
                    for( Map.Entry<Rectangle, Notable<Planet>> entry : notes.entrySet() )
                    {
                        if( entry.getKey().contains( e.getX(), e.getY() ) )
                        {
                            NoteBox<Planet> noteBox = new NoteBox<Planet>( parent, parent.getMainFrame(), entry.getValue() );
                            noteBox.setVisible( true );
                            focusRequest = false;
                        }
                    }
                }

                if( e.getClickCount() > 1 )
                {
                    Position position = getPosition( e );
                    // hit on planet?
                    if( position != null )
                    {
                        Planet planet = null;
                        try
                        {
                            planet = dataManager.getPlanetHere( position.getId() );
                        }
                        catch( SQLException e1 )
                        {
                            e1.printStackTrace();
                        }
                        if( planet != null )
                        {
                            NoteBox<Planet> noteBox = new NoteBox<Planet>( parent, parent.getMainFrame(), planet );
                            noteBox.setVisible( true );
                            focusRequest = false;
                        }
                    }
                }

                if( focusRequest )
                {
                    GalaxyComponent.this.requestFocusInWindow();
                }
            }

            @Override
            public void mousePressed( MouseEvent event )
            {
                if( event.isPopupTrigger() )
                {
                    showPopup( event );
                }
            }

            @Override
            public void mouseReleased( MouseEvent event )
            {
                if( event.isPopupTrigger() )
                {
                    showPopup( event );
                }
            }

            private void showPopup( MouseEvent event )
            {
                // hit on planet?
                Position position = getPosition( event );
                if( position != null )
                {
                    menuPlanet = null;
                    try
                    {
                        menuPlanet = dataManager.getPlanetHere( position.getId() );
                    }
                    catch( SQLException e )
                    {
                        e.printStackTrace();
                    }
                    if( menuPlanet != null )
                    {
                        JPopupMenu popup = new JPopupMenu();
                        if( menuPlanet.getOwner() != null )
                        {
                            popup.add( popupPlayer );
                            if( menuPlanet.getOwner().getAlliance() != null )
                            {
                                popup.add( popupAlliance );
                            }
                        }
                        popup.add( popupSystem );
                        popup.add( popupSpyReports );
                        popup.add( popupBattleReports );
                        popup.pack();
                        updateLabel();

                        popup.show(event.getComponent(), event.getX(), event.getY() );
                    }
                }
            }
        });
    }

    @Override
    public JToolTip createToolTip()
    {
        return new JToolTip()
        {
            private static final long serialVersionUID = 4237963755955028539L;

            public void updateUI()
            {
                GalaxyComponentToolTipUI ui = GalaxyComponentToolTipUI.createUI( this );
                ui.setDataManager( dataManager );
                setUI( ui );
            }
        };
    }


    @Override
    public String getToolTipText( MouseEvent event )
    {
        String toolTipText = null;
        Position position = getPosition( event );
        if( position != null )
        {
            toolTipText = position.toString();
        }
        return toolTipText;
    }

    private static Dimension galaxySize = new Dimension(805, 620);
    @Override
    public Dimension getPreferredSize() {
        return galaxySize;
    }

    private Position getPosition( MouseEvent event )
    {
        Position position = null;
        int mouseX = event.getX();
        int mouseY = event.getY();
        for( int planet = Position.lowerPlanetBound; planet <= Position.upperPlanetBound; planet++ )
        {
            Position p = new Position(galaxy, system, planet );
            int[] data = SystemData.getPlanetData( dataManager.getFlavor(), p );

            int baseX = data[0] + 15;
            int baseY = data[1] + 15;

            if( mouseX >= baseX && mouseX <= baseX + toolTipWidth
             && mouseY >= baseY && mouseY <= baseY + toolTipWidth
            )
            {
                position = p;
                break;
            }
        }
        return position;
    }

    protected void actionShowSpyReports( ActionEvent e )
    {
        JTabbedPane pane = parent.getMainFrame().getTabbedPane();
        int tabIndex = pane.indexOfTab( dataManager.getI18nText( "MainFrame.spyReports" ) );
        if( tabIndex >= 0 )
        {
            SpyReportSummaryPane summaryPane = ( SpyReportSummaryPane )pane.getComponentAt( tabIndex );
            summaryPane.actionNew( menuPlanet );
            pane.setSelectedIndex( tabIndex );
        }
    }

    protected void actionShowSystem( ActionEvent e )
    {
        JTabbedPane pane = parent.getMainFrame().getTabbedPane();
        int tabIndex = pane.indexOfTab( dataManager.getI18nText( "MainFrame.planets" ) );
        if( tabIndex >= 0 )
        {
            PlanetSummaryPane summaryPane = ( PlanetSummaryPane )pane.getComponentAt( tabIndex );
            summaryPane.actionNew( Position.createFromId( menuPlanet.getPosition() ) );
            pane.setSelectedIndex( tabIndex );
        }
    }

    protected void actionShowPlayer( ActionEvent e )
    {
        JTabbedPane pane = parent.getMainFrame().getTabbedPane();
        int tabIndex = pane.indexOfTab( dataManager.getI18nText( "MainFrame.planets" ) );
        if( tabIndex >= 0 )
        {
            PlanetSummaryPane summaryPane = ( PlanetSummaryPane )pane.getComponentAt( tabIndex );
            summaryPane.actionNew( menuPlanet.getOwner() );
            pane.setSelectedIndex( tabIndex );
        }
    }

    protected void actionShowAlliance( ActionEvent e )
    {
        JTabbedPane pane = parent.getMainFrame().getTabbedPane();
        int tabIndex = pane.indexOfTab( dataManager.getI18nText( "MainFrame.planets" ) );
        if( tabIndex >= 0 )
        {
            PlanetSummaryPane summaryPane = ( PlanetSummaryPane )pane.getComponentAt( tabIndex );
            summaryPane.actionNew( menuPlanet.getOwner().getAlliance() );
            pane.setSelectedIndex( tabIndex );
        }
    }

    protected void actionShowBattleReports( ActionEvent e )
    {
        JTabbedPane pane = parent.getMainFrame().getTabbedPane();
        int tabIndex = pane.indexOfTab( dataManager.getI18nText( "MainFrame.battleReports" ) );
        if( tabIndex >= 0 )
        {
            BattleReportSummaryPane summaryPane = ( BattleReportSummaryPane )pane.getComponentAt( tabIndex );
            summaryPane.actionNew( menuPlanet );
            pane.setSelectedIndex( tabIndex );
        }
    }

    protected void eventKeyPressed( KeyEvent event )
    {
        Position position = parent.getPosition();
        int galaxy = position.getGalaxy();
        int system = position.getSystem();
        switch( event.getKeyCode() )
        {
            case KeyEvent.VK_LEFT:
                if( system > Position.lowerSystemBound )
                {
                    system--;
                }
                break;
            case KeyEvent.VK_RIGHT:
                if( system < Position.upperSystemBound )
                {
                    system++;
                }
                break;
            case KeyEvent.VK_DOWN:
                if( galaxy > Position.lowerGalaxyBound )
                {
                    galaxy--;
                }
                break;
            case KeyEvent.VK_UP:
                if( galaxy < Position.getUpperGalaxyBound(dataManager.getFlavor()) )
                {
                    galaxy++;
                }
                break;
        }
        if( galaxy != position.getGalaxy() || system != position.getSystem() )
        {
            parent.setSystem( galaxy, system );
        }
    }

    protected void setSystem( int galaxy, int system )
    {
        this.galaxy = galaxy;
        this.system = system;
    }


    @Override
    public Font getFont()
    {
        UIDefaults uiDefaults = UIManager.getDefaults();
        String key = "Panel.font";
        Font font = ( Font )uiDefaults.get( key );
        return font;
    }

    protected void paintComponent(Graphics g)
    {
        super.paintComponent(g);
        if( dataManager.getActiveAccount() == null )
        {
            return;
        }

        baseFont = getFont();
        baseFont =  baseFont.deriveFont( baseFont.getSize2D() - 1.0f );
        boldFont = baseFont.deriveFont( Font.BOLD );

        // prepare background
        StringBuilder path = new StringBuilder();
        path.append( "/galaxy" );
        if( galaxy < 10 )
        {
            path.append( '0' );

        }
        path.append( galaxy );
        path.append( ".jpg" );
        ImageIcon icon = Resources.getIcon( path.toString() );
        g.drawImage( icon.getImage(), 0, 0, null );

        try
        {
            // draw sun
            drawSun( g );
            // draw planets
            drawPlanets( g );
            // decorate planets
            decoratePlanets( g );
            // update time indicator
            parent.updateLabel();
        }
        catch (SQLException ex) {
            parent.getMainFrame().showErrorInfo( ex );
        }
    }

    private void drawSun( Graphics g )
    {
        int[] data = SystemData.getSunData( dataManager.getFlavor(), new Position(galaxy, system, 0) );
        int baseX = data[0] + 13;
        int baseY = data[1] - 15;

        ImageIcon icon = Resources.getIcon( "/sun.png" );
        g.drawImage( icon.getImage(), baseX, baseY, null );
        String sun = dataManager.getI18nText( "GalaxyView.sun" );
        g.setColor( Color.white );
        g.setFont( baseFont );
        g.drawString( sun, baseX + 6, baseY + 50 );
    }

    private void drawPlanets( Graphics g ) throws SQLException
    {
        Flavor flavor = dataManager.getFlavor();
        for( int planet = Position.lowerPlanetBound; planet <= Position.upperPlanetBound; planet++ )
        {
            int[] data = SystemData.getPlanetData( flavor, new Position(galaxy, system, planet) );

            int baseX = data[0] + 20;
            int baseY = data[1] + 20;

            StringBuilder path = new StringBuilder();
            path.append( "/planet" );
            if( planet < 10 )
            {
                path.append( '0' );

            }
            path.append( planet );
            path.append( ".png" );
            ImageIcon icon = Resources.getIcon( path.toString() );
            g.drawImage( icon.getImage(), baseX, baseY, null );

            g.setColor( Color.white );
            g.drawString( Integer.toString( planet ), baseX + 25, baseY + 20 );

            // add symbols for spy and battle reports
            Planet dummy = new Planet();
            dummy.setAccount( dataManager.getActiveAccount().getId() );
            dummy.setPosition( Position.createId( galaxy, system, planet ) );
            Collection<SpyReport> spyReports = dataManager.getSpyReportsForPlanet( dummy );
            if( spyReports.size() > 0 )
            {
                icon = Resources.getIcon( "/scroll.png" );
                g.drawImage( icon.getImage(), baseX - 8, baseY + 14, null );
                SpyReport spyReport = spyReports.iterator().next();
                // save for popup
                this.spyReports.put( new Rectangle( baseX - 9, baseY + 13, 12, 12 ), spyReport );
                Map<?, Long> scanResults  = SpyReportParser.scanSpyReport( spyReport, dataManager );
                if( hasFleet( scanResults ) )
                {
                    icon = Resources.getIcon( "/fleet.png" );
                    g.drawImage( icon.getImage(), baseX + 26, baseY - 4, null );
                }
                if( hasDefense( scanResults ) )
                {
                    g.setColor( Color.white );
                    g.drawOval( baseX - 3, baseY - 3, 27, 27 );
                }
                switch( industryLevel( scanResults ) )
                {
                    case 0:
                    case 1:
                        break;
                    case 2:
                        icon = Resources.getIcon( "/silver.png" );
                        g.drawImage( icon.getImage(), baseX + 3, baseY + 2, null );
                        break;
                    case 3:
                    default:
                        icon = Resources.getIcon( "/gold.png" );
                        g.drawImage( icon.getImage(), baseX + 3, baseY + 2, null );
                        break;
                }
            }
            Collection<BattleReport> battleReports = dataManager.getBattleReportsForPlanet( dummy );
            if( battleReports.size() > 0 )
            {
                icon = Resources.getIcon( "/battle.png" );
                g.drawImage( icon.getImage(), baseX + 3, baseY + 14, null );
                // save for popup
                this.battleReports.put( new Rectangle( baseX + 2, baseY + 13, 12, 12 ), battleReports.iterator().next() );
            }
        }
    }

    private void decoratePlanets( Graphics g ) throws SQLException
    {
        Flavor flavor = dataManager.getFlavor();
        PlanetarySystem system = dataManager.getSystem( this.galaxy, this.system );
        if( system != null )
        {
            long now = System.currentTimeMillis();
            Collection<Planet> planets = system.getPlanets();
            Iterator<Planet> it = planets.iterator();
            FontMetrics metrics = g.getFontMetrics( baseFont );
            FontMetrics boldMetrics = g.getFontMetrics( boldFont );
            while( it.hasNext() )
            {
                Planet planet = it.next();
                int pl = planet.getPosition() % 100;
                int[] data = SystemData.getPlanetData( flavor, new Position( this.galaxy, this.system, pl ) );
                int baseX = data[0] + 20;
                int baseY = data[1] + 20;

                //overlay old planets
                if(planet.isEvent()) {
                    ImageIcon icon = Resources.getIcon( "/planet_event.png" );
                    g.drawImage( icon.getImage(), baseX + 3, baseY + 3, null );
                }
                if( planet.getUpdateTime() < system.getHighTime() )
                {
                    long updateInterval = now - planet.getUpdateTime();
                    Color base = GalaxyOverview.getColor( updateInterval );
                    Color color = new Color( base.getRed(), base.getGreen(), base.getBlue(), 75 );
                    g.setColor( color );
                    g.fillOval( baseX - 4, baseY - 4, 31, 31 );
                }

                Player owner = planet.getOwner();
                Status status = Status.none;
                if( owner != null && owner.getStatus() != null )
                {
                    status = owner.getStatus();
                }
                if( planet.hasAsteroid() )
                {
                    ImageIcon icon = Resources.getIcon( "/moon.png" );
                    g.drawImage( icon.getImage(), baseX - 8, baseY - 8, null );
                    // notify unfriendly asteroids
                    switch( status )
                    {
                        case enemy:
                        case noob:
                        case strong:
                            g.setColor( Status.enemy.getColor() );
                            g.drawOval( baseX - 8, baseY - 8, 9, 9 );
                    }
                    Collection<SpyReport> spyReports = dataManager.getSpyReportsForAsteroid( planet );
                    if( spyReports.size() > 0 )
                    {
                        icon = Resources.getIcon( "/scroll.png" );
                        g.drawImage( icon.getImage(), baseX - 20, baseY - 2, null );
                        SpyReport spyReport = spyReports.iterator().next();
                        // save for popup
                        this.spyReports.put( new Rectangle( baseX - 21, baseY - 3, 12, 12 ), spyReport );

                        Map<?, Long> scanResults  = SpyReportParser.scanSpyReport( spyReport, dataManager );
                        if( hasFleet( scanResults ) )
                        {
                            icon = Resources.getIcon( "/fleet.png" );
                            g.drawImage( icon.getImage(), baseX + 3, baseY - 15, null );
                        }
                        if( hasDefense( scanResults ) )
                        {
                            g.setColor( Color.white );
                            g.drawOval( baseX - 10, baseY - 10, 13, 13 );
                        }
                    }
                }
                if( planet.getDebris() != null )
                {
                    ImageIcon icon = Resources.getIcon( "/debris.png" );
                    g.drawImage( icon.getImage(), baseX + 10, baseY - 9, null );
                }
                if( planet.getNotes() != null && planet.getNotes().getText().trim().length() > 0 )
                {
                    ImageIcon icon = Resources.getIcon( "/notes-small.png" );
                    g.drawImage( icon.getImage(), baseX + 14, baseY + 14, null );
                    // save for popup
                    this.notes.put( new Rectangle( baseX + 13, baseY + 13, 12, 12 ), planet );
                }

                int posX = baseX - 2;
                int posY = baseY + 32;
                if( owner != null )
                {
                    // set color according to status
                    Color statusColor = status.getColor();
                    // extra consideration for 'locked' players
                    if( status == Status.locked )
                    {
                        Status secondaryStatus = owner.getSecondaryStatus();
                        if( secondaryStatus != null && secondaryStatus != Status.none )
                        {
                            statusColor = secondaryStatus.getColor();
                        }
                    }
                    g.setColor( statusColor );
                    // planet name is bold
                    g.setFont( boldFont );
                    g.drawString( planet.getName(), posX, posY );
                    int height = boldMetrics.getMaxAscent() + boldMetrics.getMaxDescent() + 1;
                    posY += height;
                    // name is in white parentheses
                    g.setColor( Color.white );
                    g.setFont( baseFont );
                    g.drawString( "(", posX, posY );
                    g.setColor( statusColor );
                    posX += metrics.stringWidth( "( " );
                    //g.setFont( boldFont );
                    final String title = updateTitle(owner.getTitle(), owner.getName());
                    g.drawString( title, posX, posY );
                    posX += metrics.stringWidth( title );
                    posX += metrics.stringWidth( " " );
                    g.setColor( Color.white );
                    g.setFont( baseFont );
                    g.drawString( ")", posX, posY );
                    // add points in normal font
                    g.setColor( statusColor );
                    posX = baseX - 2;
                    posY += height;
                    g.drawString( DataManager.format( owner.getPoints() ), posX, posY );
                }
            }
        }
    }

    private boolean hasFleet( Map<?, Long> scanResults )
    {
        boolean rv = false;

        for( Ship ship : Ship.ships() )
        {
            if( scanResults.containsKey( ship ) )
            {
                rv = true;
                break;
            }
        }

        return rv;
    }

    private boolean hasDefense( Map<?, Long> scanResults )
    {
        boolean rv = false;

        for( Defense defense : Defense.values() )
        {
            if( scanResults.containsKey( defense ) )
            {
                rv = true;
                break;
            }
        }

        return rv;
    }

    private int industryLevel( Map<?, Long> scanResults )
    {
        int rv = 0;

        for( Building building : EnumSet.range( Building.ironMine, Building.spiceMine ) )
        {
            Long level = scanResults.get( building );
            if( level != null )
            {
                int value = (level.intValue() - 1) / 10;
                if( rv < value)
                {
                    rv = value;
                }
            }
        }
        return rv;
    }

    private void updateLabel()
    {
        popupAlliance.setText( dataManager.getI18nText( "Popup.showAlliance" ) );
        popupPlayer.setText( dataManager.getI18nText( "Popup.showPlayer" ) );
        popupSystem.setText( dataManager.getI18nText( "Popup.showSystem" ) );
        popupSpyReports.setText( dataManager.getI18nText( "Popup.spyReports" ) );
        popupBattleReports.setText( dataManager.getI18nText( "Popup.battleReports" ) );
    }

    static String updateTitle(String title, String name) {
        if(title.isEmpty() || title.contains("<div")) {
            title = "%name%";
        }
        return title.replace("%name%", name);
    }

    public void update() {
        revalidate();
        repaint();
    }


}
