/****************************************
 *  COPYRIGHT (C) 2010
 *  Holger Graf
 ****************************************/

package siarchive.imports;

import java.io.IOException;
import java.sql.SQLException;
import java.util.Collection;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.fasterxml.jackson.jr.ob.JSON;
import siarchive.DataManager;
import siarchive.components.Cost;
import siarchive.components.Position;
import siarchive.components.Status;
import siarchive.json.JSONTags;
import siarchive.json.galaxy.GalaxyBean;
import siarchive.json.galaxy.PlanetBean;
import siarchive.json.galaxy.UserBean;
import siarchive.persistence.Alliance;
import siarchive.persistence.Planet;
import siarchive.persistence.Player;

/**
 * @author graf
 *
 */
public class GalaxyImporterNT
{
    private DataManager dataManager;

    private Pattern areaPattern = Pattern.compile( "<\\s*textarea.*?>(.*?)</\\s*textarea>", Pattern.DOTALL );


    public GalaxyImporterNT( DataManager dataManager )
    {
        this.dataManager = dataManager;
    }

    public Position scan( String input ) throws SQLException
    {
        if(input.indexOf("textarea") >= 0) {
            // HTML source input
            Matcher areaMatcher = areaPattern.matcher( input );
            while( areaMatcher.find() ) {
                input = areaMatcher.group( 1 );
                if(input.indexOf(JSONTags.reportType_GalaxyOverview) >= 0) {
                    return scanJSON(input);
                }
            }
        }
        else {
            // clean JSON input
            return scanJSON(input);
        }
        return new Position(0, 0, 0);
    }

    private Position scanJSON( String input ) throws SQLException {
        long now = System.currentTimeMillis();
        int galaxy = 0;
        int system = 0;
        try {
            GalaxyBean galaxyBean = JSON.std.beanFrom(GalaxyBean.class, input);
            galaxy = galaxyBean.getPosition().getGalaxy();
            system = galaxyBean.getPosition().getSystem();
            // collect old data for orphan check
            Set<Player> owners = new HashSet<Player>();
            Set<Alliance> alliances = new HashSet<Alliance>();
            if( dataManager.getSystem( galaxy, system ) != null ) {
                for( Planet planet : dataManager.getSystem( galaxy, system ).getPlanets() ) {
                    Player owner = planet.getOwner();
                    if( owner != null ) {
                        owners.add( owner );
                        Alliance alliance = owner.getAlliance();
                        if( alliance != null ) {
                            alliances.add( alliance );
                        }
                    }
                }
            }

            // get recognized planets and remove from oldPlanets
            savePlanets( galaxyBean.getPlanets().values(), now );

            // remove orphans
            dataManager.removeOrphanedPlayers( owners );
            dataManager.removeOrphanedAlliances(alliances);
            dataManager.commit();
        }
        catch( IOException e1 ) {
            e1.printStackTrace();
        }
        return new Position( galaxy, system, 1 );
    }

    private void savePlanets( Collection<PlanetBean> planets, long now ) throws SQLException {
        for(PlanetBean planetBean : planets) {
            final UserBean userBean = planetBean.getUser();
            final int position = Position.createId(planetBean.getCoords());
            final Long account = dataManager.getActiveAccountId();
            Planet planet = new Planet();
            planet.setAccount(account);
            planet.setPosition(position);
            planet.setEvent(planetBean.isIsEventPlanet());
            Map<String, Long> rubble = planetBean.getRubblefield();
            if(rubble != null ) {
                planet.setDebris(new Cost(rubble.get("i"), rubble.get("m"), rubble.get("k"), rubble.get("s")));
            }
            planet.setUpdateTime( now );
            if(userBean != null) {
                planet.setName(planetBean.getName());
                planet.setAsteroid(planetBean.isHasAstro());
                // do not overwrite existing notes
                planet.setRetainNotes( true );

                Player owner = new Player();
                owner.setAccount( account );
                owner.setName(userBean.getNickname());
                owner.setTitle(userBean.getTitle());
                owner.setPlayerId(userBean.getId());
                // admins don't have ranking
                owner.setPoints(userBean.getRanking());
                owner.setStatus(convertStatus(userBean.getStatus()));
                owner.setSecondaryStatus((userBean.getStatus().getSecondary() != null)
                    ? convertStatus(userBean.getStatus().getSecondary())
                    : Status.none);

                // add alliance
                Alliance alliance = new Alliance();
                alliance.setAccount( account );
                // the empty alliance will get ID 0
                alliance.setAllianceId( 0 );
                alliance.setName( "" );
                alliance.setUpdateTime( now );
                // do not overwrite existing notes
                alliance.setRetainNotes( true );
                if(userBean.getAlliance() != null) {
                    alliance.setAllianceId(userBean.getAlliance().getId());
                    alliance.setName(userBean.getAlliance().getName());
                    alliance.setTag(userBean.getAlliance().getTag());
                }
                // persist intermediate objects to add persistent references to planet
                // so they won't be doubled after restart
                alliance = dataManager.save( alliance );
                owner.setAlliance( alliance );
                owner.setUpdateTime( now );
                // do not overwrite existing notes
                owner.setRetainNotes( true );
                // persist intermediate objects to add persistent references to planet
                // so they won't be doubled after restart
                owner = dataManager.save( owner );

                planet.setOwner( owner );
                // save recognized planets
                dataManager.save( planet);
            }
            else {
                if(planet.getDebris() != null || planet.isEvent()) {
                    dataManager.save( planet);
                }
                else {
                    planet = dataManager.getPlanetHere(position);
                    if(planet != null) {
                        dataManager.delete(planet);
                    }
                }
            }
        }
    }

    private siarchive.components.Status convertStatus(siarchive.json.galaxy.Status statusJson) {
        siarchive.components.Status status = siarchive.components.Status.none;;
        switch(statusJson) {
            case locked:
                status = siarchive.components.Status.locked;
                break;
            case admin:
                status = siarchive.components.Status.operator;
                break;
            case friendly:
                status = siarchive.components.Status.friendly;
                break;
            case vacationMode:
                status = siarchive.components.Status.vacation;
                break;
            case inactive:
                status = siarchive.components.Status.inactive;
                break;
            case enemy:
                status = siarchive.components.Status.enemy;
                break;
            case bnd:
                status = siarchive.components.Status.bnd;
                break;
            case nap:
                status = siarchive.components.Status.nap;
                break;
            case wing:
                status = siarchive.components.Status.wing;
                break;
            case noob:
                status = siarchive.components.Status.noob;
                break;
            case strong:
                status = siarchive.components.Status.strong;
                break;
            case own:
                status = siarchive.components.Status.own;
                break;
            case spectral:
                status = siarchive.components.Status.spectral;
                break;
            case noUser:
            default:
                status = siarchive.components.Status.none;
                break;
        }
        return status;
    }
}
