/****************************************
 *  COPYRIGHT (C) 2012
 *  Holger Graf
 ****************************************/
package siarchive.persistence.dao;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import siarchive.MainFrame;
import siarchive.components.Priority;
import siarchive.persistence.Report;


/**
 * @author graf
 *
 */
public abstract class ReportDao<T extends Report<T>> extends BaseDao<T>
{
    public final static String table = "report";

    protected final static String ORDERBYCREATIONTIMEASC = " ORDER BY CREATETIME ASC";

    public ReportDao()
    {
        super( 500 );
    }

    private final String COUNT = "SELECT COUNT(*) FROM " + getTable() + " WHERE KIND = ?";
    public int count() throws SQLException
    {
        PreparedStatement count = getConnection().prepareStatement( COUNT );
        count.setString( 1, getKind() );
        int records = 0;
        ResultSet rs = count.executeQuery();
        if (rs.next())
        {
            records = rs.getInt(1);
        }
        rs.close();
        count.close();
        return records;
    }
    
    private final String COUNTBYACCOUNT = "SELECT COUNT(*) FROM " + getTable() + " WHERE ACCOUNT = ? AND KIND = ?";
    public int countByAccount(Long account) throws SQLException
    {
        PreparedStatement count = getConnection().prepareStatement( COUNTBYACCOUNT );
        int records = 0;
        count.setLong( 1, account );
        count.setString( 2, getKind() );
        ResultSet rs = count.executeQuery();
        if (rs.next())
        {
            records = rs.getInt(1);
        }
        rs.close();
        count.close();
        return records;
    }

    private final String FINDBYACCOUNT = "SELECT * FROM " + getTable() + " WHERE ACCOUNT = ? AND KIND = ?" + ORDERBYCREATIONTIMEASC;
    public List<T> findByAccount(Long account) throws SQLException
    {
        PreparedStatement find = getConnection().prepareStatement( FINDBYACCOUNT, ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
        find.setLong( 1, account );
        find.setString( 2, getKind() );
        ResultSet resultSet = find.executeQuery();
        List<T> list = createDTO( resultSet ); 
        resultSet.close();
        find.close();
        return list;
    }

    private final String FINDBYACCOUNTANDCREATIONTIME = "SELECT * FROM " + getTable() + " WHERE ACCOUNT = ? AND KIND = ? AND CREATETIME = ?" + ORDERBYCREATIONTIMEASC;
    public List<T> findByCreationTime(Long account, long creationTime) throws SQLException
    {
        PreparedStatement find = getConnection().prepareStatement( FINDBYACCOUNTANDCREATIONTIME, ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
        find.setLong( 1, account );
        find.setString( 2, getKind() );
        find.setLong( 3, creationTime );
        ResultSet resultSet = find.executeQuery();
        List<T> list = createDTO( resultSet ); 
        resultSet.close();
        find.close();
        return list;
    }

    private final String FINDBYACCOUNTPOSITIONANDCREATIONTIME = "SELECT * FROM " + getTable() + " WHERE ACCOUNT = ? AND KIND = ? AND POSITION = ? AND CREATETIME = ?" + ORDERBYCREATIONTIMEASC;
    public T find(Long account, int position, long creationTime) throws SQLException
    {
        PreparedStatement find = getConnection().prepareStatement( FINDBYACCOUNTPOSITIONANDCREATIONTIME, ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
        find.setLong( 1, account );
        find.setString( 2, getKind() );
        find.setInt( 3, position );
        find.setLong( 4, creationTime );
        ResultSet resultSet = find.executeQuery();
        List<T> list = createDTO( resultSet ); 
        resultSet.close();
        find.close();
        return getFirst( list );
    }
    
    private final String FINDBETWEENPOSITION = "SELECT * FROM " + getTable() + " WHERE ACCOUNT = ? AND KIND = ? AND POSITION >= ? AND POSITION <= ?" + ORDERBYCREATIONTIMEASC;
    public List<T> findByPosition(Long account, int startPosition, int stopPosition) throws SQLException
    {
        PreparedStatement find = getConnection().prepareStatement( FINDBETWEENPOSITION, ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
        find.setLong( 1, account );
        find.setString( 2, getKind() );
        find.setInt( 3, startPosition );
        find.setInt( 4, stopPosition );
        ResultSet resultSet = find.executeQuery();
        List<T> list = createDTO( resultSet ); 
        resultSet.close();
        find.close();
        return list;
    }
    
    private final String FINDBETWEENPOSITIONANDPRIORITY = "SELECT * FROM " + getTable() + " WHERE ACCOUNT = ? AND KIND = ? AND CREATETIME >= ? AND CREATETIME <= ? AND PRIORITY = ?" + ORDERBYCREATIONTIMEASC;
    public List<T> findByPositionAndPriority(Long account, int startPosition, int stopPosition, Priority priority) throws SQLException
    {
        PreparedStatement find = getConnection().prepareStatement( FINDBETWEENPOSITIONANDPRIORITY, ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
        find.setLong( 1, account );
        find.setString( 2, getKind() );
        find.setLong( 3, startPosition );
        find.setLong( 4, stopPosition );
        find.setString( 5, priority.name() );
        ResultSet resultSet = find.executeQuery();
        List<T> list = createDTO( resultSet ); 
        resultSet.close();
        find.close();
        return list;
    }

    private final String FINDBYTIME = "SELECT * FROM " + getTable() + " WHERE ACCOUNT = ? AND KIND = ? AND CREATETIME >= ? AND CREATETIME <= ?" + ORDERBYCREATIONTIMEASC;
    public List<T> findByTime(Long account, long startTime, long stopTime) throws SQLException
    {
        PreparedStatement find = getConnection().prepareStatement( FINDBYTIME, ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
        find.setLong( 1, account );
        find.setString( 2, getKind() );
        find.setLong( 3, startTime );
        find.setLong( 4, stopTime );
        ResultSet resultSet = find.executeQuery();
        List<T> list = createDTO( resultSet ); 
        resultSet.close();
        find.close();
        return list;
    }
    
    private final String FINDBYTIMEANDPRIORITY = "SELECT * FROM " + getTable() + " WHERE ACCOUNT = ? AND KIND = ? AND CREATETIME >= ? AND CREATETIME <= ? AND PRIORITY = ?" + ORDERBYCREATIONTIMEASC;
    public List<T> findByTimeAndPriority(Long account, long startTime, long stopTime, Priority priority) throws SQLException
    {
        PreparedStatement find = getConnection().prepareStatement( FINDBYTIMEANDPRIORITY, ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
        find.setLong( 1, account );
        find.setString( 2, getKind() );
        find.setLong( 3, startTime );
        find.setLong( 4, stopTime );
        find.setString( 5, priority.name() );
        ResultSet resultSet = find.executeQuery();
        List<T> list = createDTO( resultSet ); 
        resultSet.close();
        find.close();
        return list;
    }

    public List<T> find(Long account, int position)  throws SQLException
    {
        return find( account, position, false );
    }

    private final String FINDBYPOSITION = "SELECT * FROM " + getTable() + " WHERE ACCOUNT = ? AND KIND = ? AND POSITION = ? AND ISASTEROID = ?" + ORDERBYCREATIONTIMEASC;
    public List<T> find(Long account, int position, boolean isAsteroid)  throws SQLException
    {
        PreparedStatement find = getConnection().prepareStatement( FINDBYPOSITION, ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
        find.setLong( 1, account );
        find.setString( 2, getKind() );
        find.setInt( 3, position );
        find.setBoolean( 4, isAsteroid );
        ResultSet resultSet = find.executeQuery();
        List<T> list = createDTO( resultSet ); 
        resultSet.close();
        find.close();
        return list;
    }
    
    protected T fillBaseDTO(T report, ResultSet resultSet) throws SQLException
    {
        report.setId( resultSet.getLong( "ID" ) );
        report.setAccount( resultSet.getLong( "ACCOUNT" ) );
        report.setCreationTime( resultSet.getLong( "CREATETIME" ) );
        report.setPosition( resultSet.getInt( "POSITION" ) );
        report.setAsteroid( resultSet.getBoolean( "ISASTEROID" ) );
        report.setFlag( MainFrame.parsePriorityAttribute( resultSet.getString( "PRIORITY" ) ) );
        report.setData( resultSet.getString( "REPORTDATA" ) );
        report.setNotes(getNote( resultSet.getString( "NOTES" ) ) );
        return report;
    }
    
    @Override
    protected String getTable()
    {
        return table;
    }

    
    @Override
	protected T findUnique(T report) throws SQLException 
	{
		return find(report.getAccount(), report.getPosition(), report.getCreationTime());
	}

    
    @Override
	protected PreparedStatement createStatement(T report) throws SQLException 
	{
        PreparedStatement create = reportCreateStatement( report );
        create.setLong( 1, report.getAccount() );
        create.setLong( 2, report.getCreationTime() );
        create.setInt( 3, report.getPosition() );
        create.setBoolean( 4, report.isAsteroid() );
        String flag = (report.getFlag() != null) ? report.getFlag().name() : null;
        create.setString( 5, flag );
        create.setString( 6, report.getData() );
        create.setString( 7, getNote( report.getNotes() ) );
		return create;
	}

	@Override
	protected PreparedStatement updateStatement(T report) throws SQLException 
	{
        PreparedStatement update = reportUpdateStatement( report );
        update.setLong( 1, report.getAccount() );
        update.setLong( 2, report.getCreationTime() );
        update.setInt( 3, report.getPosition() );
        update.setBoolean( 4, report.isAsteroid() );
        String flag = (report.getFlag() != null) ? report.getFlag().name() : null;
        update.setString( 5, flag );
        update.setString( 6, report.getData() );
		return update;
	}

    protected abstract String getKind();
    protected abstract PreparedStatement reportCreateStatement(T report) throws SQLException;
    protected abstract PreparedStatement reportUpdateStatement(T report) throws SQLException;
}
